'----------------------------------------------------------------------------------------
'    Name:  cgi_V002.inc
'    Type:  TIGER-BASIC(tm) Source Code
' Purpose:  cgi programms for embedded webserver
'
'
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tigers in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (english)
'	email: support@wilke.de              (german)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (english)
'	www.wilke.de                         (german)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------
'
' required Hardware:
' Ethernet (Prototyping) Board with Tiny Tiger TCN 1/4 and EM-01
'
' graphic LCD for debug output
'
' here are different cgi programms, which can be started from a client 
' outside the Tiger.
'
'
'----------------------------------------------------------------------------------------

'*********************************************************************************
' This sub starts a cgi application. The name of the cgi programm must exist
' in the switch...case instruction.
'---------------------------------------------------------------------------------
' Parameters:
' - uri$: URI of HTTP request
' - retval: succes of function (TRUE,FALSE) 
'*********************************************************************************
sub startCGI(var string uri$;var byte retval)
  long pos
  string cgi_name$
  
  pos = instr( cgi_directory , uri$ , 0 , 4 )           ' search for cgi directory
  if pos < 0 then                                       ' is there really a cgi directory
    retval = FALSE                                      ' no? --> NOT successful
    return                                              ' exit function
  endif                                                 ' end of if pos < 0
  pos = pos + len(cgi_directory)                        ' get position after directory name
  cgi_name$ = right$(uri$, len(uri$) - pos - 1)         ' get cgi name
  switch cgi_name$                                      ' compare cgi name
    case "toggleLED":                       
      answer_document$ = ""                             ' no entity in response
      retval = TRUE                                     ' success
      content_type$ = html                              ' set content type
      statuscode$ = HTTP_OK_NORESPONSE                  ' statuscode: no response(entity)
      call blinkLeds(VARLIST$)                          ' change LED
    case "resetEnc":
      call resetEnc                                     ' reset encoder value
      call generateHTMLCode(retval)                     ' generate new HTML site dynamic
    case "resetKeys":
      answer_document$ = ""                             ' no entity in response
      retval = TRUE                                     ' success
      content_type$ = html                              
      statuscode$ = HTTP_OK_NORESPONSE                  ' statuscode: no response(entity)
      call resetKeys                                    ' reset keys
    case "rand":
      call randomizeLED                                 ' randomize staurus of LEDs
      call generateHTMLCode(retval)                     ' generate new HTML site dynamic
    default: 
      retval = FALSE                                    ' no success
  endswitch
end


'*****************************************************************************************
' this sub generates the HTML Code, which is required for the browser. 
' The HTML Code will be generated and the actual settings are given by parameter.
' The dynamic generated elements are:
' - status of keys
' - status of LEDs
' - encoder value
'-----------------------------------------------------------------------------------------
' Parameters:
' - retval: successful or not (TRUE,FALSE)
'***************************************************************************************** 
sub generateHTMLCode(var byte retval)
  byte check
  byte x
  long pos
  string doc_namex$(58)
  string search$
  string sInsert$(77)
  long freeStack
  long errc
  doc_namex$ = let$ ( "main_page_V002.html" )                                    ' name of default document(incomplete)
                       
  call getSendDocument( answer_document$, doc_namex$, content_type$,0, check )   ' get document from flash memory
  if check = FALSE then                                                          ' successful?
    retval = FALSE                                                               ' NOT successful
    return                                                                       ' exit function
  endif
  ' search in document gor right place to insert java-Code
  ' search for "var zustand = new Array(5);"
  sInsert$ = "<13><10>"                                                          ' init sInsert$
  for x = 0 to 4                                                                 ' for all 5 keys
    
    sInsert$ = sInsert$ + "zustand[" + right$ ( str$ ( x ) , 1 ) + "]=" + &
    right$ ( str$(sgn(pressedKeys bitand ( 01H shl x ) ) ) , 1 ) + ";<13><10>"   ' insert this text: zustand[x]=y;
    
    switch pressedKeys bitand ( 01H shl x )                                      ' what text to search?           
      case 01H:
        search$ = "<22H>lightState0<22H> value = <22H>1"
      case 02H:
        search$ = "<22H>lightState1<22H> value = <22H>1"
      case 04H:
        search$ = "<22H>lightState2<22H> value = <22H>1"
      case 08H:
        search$ = "<22H>lightState3<22H> value = <22H>1"
      case 10H:
        search$ = "<22H>lightState4<22H> value = <22H>1"
      default:  ' button unpressed, change nothing
        search$ = ""
    endswitch
    if len(search$) > 0 then	' let's change source code 
      call search( answer_document$ , search$ , pos )                          ' search text
      if pos >= 0 then                                                         ' text found?  
        answer_document$ = stos$ ( answer_document$ , pos + len ( search$ ) - 1 , "0" , 1 )  ' change parameters in HTML source code 
      endif
    endif
  next
  ' insert generated Code to HTML-Page
  search$ = "var zustand = new Array(5);"                       ' text to search
  call search( answer_document$ , search$ , pos )               ' serach text
  if pos < 0 then                                               ' text found?
    retval = FALSE                                              ' NOT successful
    return                                                      ' exit function
  endif
  answer_document$ = let$ ( left$ ( answer_document$ , pos + len ( search$ ) ) + sInsert$ + &
  right$ ( answer_document$ , len ( answer_document$ ) - pos - len ( search$ ) ) )    ' insert text sInsert$ after
  '                                                                                   ' search$
  ' The Encoder-Value must be showm on Site, too
  search$ = "var encVal;"                                                             ' text to search
  call search ( answer_document$ , search$ , pos )                                    ' search text
  if pos < 0 then                                                                     ' text found?
    retval = FALSE                                                                    ' NOT successful
    return                                                                            ' exit function
  endif
  get #ENC, #0, 0, enc_val    '' #0 = read only ( not reset )
  answer_document$ = let$ ( left$ ( answer_document$ , pos + len ( search$ ) ) + "<13><10>encVal=" + str$ ( enc_val ) + &
  ";" + right$ ( answer_document$ , len ( answer_document$ ) - pos - len ( search$ ) ) )    ' insert generated code here
  if enc_val < 0 then                                                                 
    search$ = "Encoder:"                                                              ' text to search     
  call search ( answer_document$ , search$ , pos )                                    ' search text
  if pos < 0 then
    retval = FALSE                                                                    ' NOT successful
    return                                                                            ' exit function
  endif
  answer_document$ = let$ ( left$ ( answer_document$ , pos + len ( search$ ) ) + &
  " -" + right$ ( answer_document$ , len ( answer_document$ ) - pos - len ( search$ ) ) ) ' insert generated code here
  endif
  retval = TRUE                                                                      ' success
end

'***************************************************************************************
' This sub is for searching a text in a document and get the position of the text
'---------------------------------------------------------------------------------------
' Parameters:
' - doc$:    document to search in
' - search$: text to find in doc$
' - erg: return value, is position of the text or -1 if the text was not found
'***************************************************************************************
sub search(var string doc$; string search$; var long erg)
  long len_s, len_d, pos
  len_s = len(search$)                         ' length of text to search
  len_d = len(doc$)                            ' length of document to search in
  pos = 0                                      ' init pos
  erg = -1                                     ' init erg
  while erg < 0 and pos <= len_d               ' repeat until text was found or end of document 
    erg = instr(search$ , doc$ , pos , 4 )     ' search for text
    pos = pos + 255 - len_s                    ' inrease pos
  endwhile
end

'***************************************************************************************
' this sub processes the status of the LED
' if a key was pressed in Browser or direct at starter kit, this function toggles
' the belonging LED
'---------------------------------------------------------------------------------------
' Parameters:
' - varlist$: this is the list of variables (1 variable) from the URI
'***************************************************************************************
sub blinkLeds(var string varlist$)
  long param
  param = val_num(varlist$)                  ' get numeric value of string number
  switch param
  case 8:		'up
    LEDs(0) = LEDs(0) bitxor 01000000b
    pressedKeys = pressedKeys bitxor 01h
  case 5:
    LEDs(2) = LEDs(2) bitxor 255
    pressedKeys = pressedKeys bitxor 04H
  case 2:
    LEDs(0) = LEDs(0) bitxor 10000000b
    pressedKeys = pressedKeys bitxor 10H
  case 4:
    LEDs(1) = LEDs(1) bitxor 00000001b
    pressedKeys = pressedKeys bitxor 02H
  case 6:
    LEDs(1) = LEDs(1) bitxor 00000010b
    pressedKeys = pressedKeys bitxor 08H
  default:
  endswitch
  
  ' output LEDs
  if LEDs(2) = 0 then
    out 3, 11000000b, 0
    out 7, 00000011b, 0     
  else
    ' special LED
    out 3, 11000000b, LEDs(0)
    out 7, 00000011b, LEDs(1)
  endif
end

'***************************************************************************************
' This sub resets the Encodervalue to zero
'***************************************************************************************
sub resetEnc()
  long x
  get #ENC, #1, 0, x    '' #0 = read out and reset
end

'****************************************************************************************
' this sub resets the keys and the LED
'****************************************************************************************
sub resetKeys()
  byte x
  pressedKeys = 0  ' reset all Keys tu unpressed
  for x = 0 to 2
    LEDs(x) = 255  'reset all LED-values to OFF
  next
  out 3, 11000000b, LEDs(0)
  out 7, 00000011b, LEDs(1)
end

'***************************************************************************************
' This sub randomizes the status of the LED, resp. of the keys
' --------------------------------------------------------------------------------------
' global Parameters:
' - LEDs(array)
'***************************************************************************************
sub randomizeLED()
  long randValue
  randomize
  pressedKeys = rnd(0) bitand 1FH             ' randomize the status of the keys
  
  ' set the value of the LEDs
  LEDs(0) = LEDs(0) bitxor ( ( ( pressedKeys bitand 01H bitxor 01H ) shl 6 ) bitxor ( LEDs(0) bitand 40H ) )
  LEDs(0) = LEDs(0) bitxor ( ( ( pressedKeys bitand 10H bitxor 10H ) shl 3 ) bitxor ( LEDs(0) bitand 80H ) )
  LEDs(1) = LEDs(1) bitxor ( ( ( pressedKeys bitand 02H bitxor 02H ) shr 1 ) bitxor ( LEDs(1) bitand 01H ) )
  LEDs(1) = LEDs(1) bitxor ( ( ( pressedKeys bitand 08H bitxor 08H ) shr 2 ) bitxor ( LEDs(1) bitand 02H ) )
  if pressedKeys bitand 04H > 0 then 
    LEDs(2) = 0
  else
    LEDs(2) = 255
  endif
  
  'running light
  loop 10
    out 3, 10000000b, 255         'down
    out 7, 00000001b, 00000000b	' left
    wait_duration 100
    out 7, 00000001b, 255         ' left
    out 3, 01000000b, 00000000b	'up
    wait_duration 100
    out 3, 01000000b, 255  	'up
    out 7, 00000010b, 00000000b   'right
    wait_duration 100
    out 7, 00000010b, 255         'right
    out 3, 10000000b, 00000000b   'down
    wait_duration 100
  endloop
  
  ' output to LED
  if LEDs(2) = 0 then
    out 3, 11000000b, 0
    out 7, 00000011b, 0     
  else
    out 3, 11000000b, LEDs(0)
    out 7, 00000011b, LEDs(1)
  endif
  
end

