'----------------------------------------------------------------------------------------
'    Name:  EM_IR_V004.inc
'    Type:  TIGER-BASIC(tm) Source Code
' Purpose:  Callbacks for HTTP server
'
'
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tigers in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (english)
'	email: support@wilke.de              (german)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (english)
'	www.wilke.de                         (german)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------
'
' required Hardware:
' Ethernet (Prototyping) Board with Tiny Tiger TCN 1/4 and EM-01
'
' graphic LCD for debug output
'
'----------------------------------------------------------------------------------------
' variables:
'
' - received_buffer_in_use(ARRAY):
'     Index = Buffer Nr
'     Variable = Socket Nr
'     ______________________
'    |      Buffer Nr       |
'    |______________________|
'    |      Socket Nr       |
'    |______________________|
'
'       FREE = 0
'
' - socket_info(ARRAY):
'     Index1 = Socket Nr
'     Index2 = unused
'     Variable1 = Buffer Nr
'     Variable2 = unused
'
'      ______________________
'     |      Socket Nr       |
'  ___|______________________|
' | 0 |      Buffer Nr       |
' |___|______________________| 
' | 1 |      unused          |
' |___|______________________| 
'
'
'----------------------------------------------------------------------------------------


#ifndef SERVER_CLBKS_INC
#define SERVER_CLBKS_INC


byte bQuitConnectionSignal
byte bSocketAccepted
byte bTcpClosedByRemoteHost
'' byte bPppIsUp
byte bPppIsDown


#ifdef TS_ESTABLISH_CB_ENABLED
byte bConnEstablished
#endif	'' TS_ESTABLISH_CB_ENABLED


'***********************************************************************************+
' This task is run, when a new socket/connection was accepted. Following variables
' are set:
'
' - lActAcceptSocket:     socket number
' - wActAcceptSABlockLen: length of sActAcceptSABlock$
' - sActAcceptSABlock$:   information about the socket
'************************************************************************************
task OnAccept
  byte blSaLength, blSaFamily
  word wlSaPort
  long llSaAddress
  call vParseSockAddrBlock( sActAcceptSABlock$, blSaLength, blSaFamily, wlSaPort, llSaAddress ) 'decode information about socket
  ' search free buffer
  byte x
  x = 1
  while not received_buffer_in_use(x) = FREE         ' as long there is a free buffer
    x = x + 1                                        ' increase buffer number
    if x > maxSockets then                           ' last buffer?
      call textBox("no free buffer")                 ' output to LCD
    endif
  endwhile
  received_buffer_in_use(x) = lActAcceptSocket       ' save socket number
  socket_Info(lActAcceptSocket,0) = x                ' save buffer number
  socket_time_out(x - 1) = 0  ' Timeout for socket   ' reset socket timeout to 0
  
  bSocketAccepted = TRUE                             ' set flag that new socket was accepted
end


'***********************************************************************************
' This task is called, when data was received
' following parameters was set:
'
' - lActDataSocket:  socket number, where data has received
' - lActDataSize:    size of data in buffer
' - sActDataBuffer$: data itself
'***********************************************************************************
task OnData
  long llSentBytesNum
  string tmp$
  byte BufferNr
  BufferNr = socket_Info(lActDataSocket,0)                        ' get buffer number
  received$(BufferNr) = received$(BufferNr) + sActdataBuffer$     ' save data in buffer
  socket_time_out(BufferNr - 1) = 0                               ' reset timout for
  receivedFlag = receivedFlag BITOR (0001H SHL BufferNr)          ' set flag that data was received
  #cm
  call textBox("data received")
  tmp$ = str$(lActDataSocket)
  call textBox(tmp$)
  #endcm
end


''lActRemoteCloseSocket
task OnRemoteClose
  byte BufferNr
  BufferNr = socket_Info(lActRemoteCloseSocket,0)
  call textBox("socket closed")
  
  received_buffer_in_use(BufferNr) = FREE
  received$(BufferNr) = ""
  socket_info(lActRemoteCloseSocket , 0 ) = FREE 

' closedSockets = closedSockets BITOR (0001H SHL BufferNr)
end


#cm
task OnPppUp
  bPppIsUp = TRUE
end
#endcm


task OnPppDown
  bPppIsDown = TRUE
end


#ifdef TS_UDP_ENABLED
'' lActRecvFromAddress, wActRecvFromPort,
'' wActRecvFromFlags, lActRecvFromDataSize, sActRecvFromDataBuffer$(RECV_PACK_MAX_LEN)
task OnDataRecvFrom
end
#endif


#ifdef TS_ESTABLISH_CB_ENABLED
'' lActEstSocket, wActEstLocalSABlockLen, sActEstLocalSABlock$,
'' wActEstRemoteSABlockLen, sActEstRemoteSABlock$
task OnEstablish
  byte blSaLength, blSaFamily
  word wlSaLocalPort, wlSaRemotePort
  long llSaLocalAddress, llSaRemoteAddress
  call vParseSockAddrBlock( sActEstLocalSABlock$, blSaLength, blSaFamily, wlSaLocalPort, llSaLocalAddress )
  call vParseSockAddrBlock( sActEstRemoteSABlock$, blSaLength, blSaFamily, wlSaRemotePort, llSaRemoteAddress )
  bConnEstablished = TRUE
end
#endif	'' TS_ESTABLISH_CB_ENABLED

#endif  '' SERVER_CLBKS_INC

