'----------------------------------------------------------------------------------------
'    Name:  HTTP_Protokoll_V007.inc
'    Type:  TIGER-BASIC(tm) Source Code
' Purpose:  implements the HTTP protocoll
'
'
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tigers in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (english)
'	email: support@wilke.de              (german)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (english)
'	www.wilke.de                         (german)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------
'
' required Hardware:
' Ethernet (Prototyping) Board with Tiny Tiger TCN 1/4 and EM-01
'
' graphic LCD for debug output
'
' The functions of this file is to read out a HTTP request and generate and send
' back a response to the client
'
' functions:
'   - seperateHTTPRequest(var byte check)
'       This functions seperates a text into 2 parts, who are seperated with
'       an empty line
'   - searchEmptyLine(var string s$; var byte check)
'       This function is searching for an empty line in a text (CrLf CrLf)
'   - request_line(var string header$; var byte check)
'       This function analyses the request line of a HTTP request
'   - seperateVarListFromUri(var string uri$,varList$)
'       This function seperates the variables from the URI.
'       They are seperated with a "?"
'   - sendHTTP(var string document$, content_type$; byte typ; long socket; var byte check)
'       This function is to send an answer to a HTTP request. After the request was analyzied
'       and the belonging document was found, you can startthis function to
'       generate the complete HTTP response.
'
'
'----------------------------------------------------------------------------------------


'------------------------ global variables --------------------------------------------

byte http_flags
' BIT0 = 1 : amswer document not saved complete in RAM


'***********************************************************
' in this sub the header of the HTTP-message and the entity
' is saved in 2 different variables
' these 2 parts are seperated witch a line consisting only of
' a CrLf -> at the end of every line is a CrLf, so we search
' for a double CrLf
'-----------------------------------------------------------
' parameters:
' request$: HTTP-request
' header$: here is saved the header of the request
' entity$: here is saved the entity
' check: see, if an error has occured
'***********************************************************
sub seperateHTTPRequest(var byte check)
  long position						'position of empty line
  long position_tmp
  position_tmp = -251                                   ' initialize var
  position = -1                                         ' initialize var
  while position < 0	  				'while Line not found
    position_tmp = position_tmp + 251                   ' search on longer strings
    if position_tmp >= len(request$) then		'end of String
      check = FALSE
      return					'empty line not found
    endif
    position = instr(Tb2xCrLf, request$, position_tmp, 0) 'search for empty line
  endwhile
  'empty line found
  header$ = left$(request$, position)		'left from empty line
  'entity$ = right$(request$, len(request$) - (position + 4))  'right from empty line
  position = position + 4
  call delfirstChar(position, request$)
  check = TRUE
end

'***********************************************************************************
' this sub searches for an empty line
' The header of a HTTP request end with an empty line
' --------------------------------------------------------------------------------
' parameters:
'   s$:    text to search in
'   check: TRUE = empty line found
'          FALSE = empty line NOT found
'***********************************************************************************
sub searchEmptyLine(var string s$; var byte check)
  long position						'position of empty line
  long position_tmp
  position_tmp = -251
  position = -1
  while position < 0	  				'while Line not found
    position_tmp = position_tmp + 251
    if position_tmp >= len(s$) then		'end of String
      check = FALSE
      return					'empty line not found
    endif
    position = instr(Tb2xCrLf, s$, position_tmp, 0) 'search for empty line
  endwhile
  'empty line found
  check = TRUE
end

'********************************************************************************
' this sub works on the first line of the request, the so called
' request line. There are several possibilities.
' 1. GET   2. HEAD   3. POST
' parameters:
' - header$:      head of HTTP request
' - flag_tab$:    a string of size 256(dummy)[because of stack size]
' - result_check_keyword$: 	"	(128 Byte)     "
' - check:        result
'   CASE: GET
'      ->vars: URI$, HTTPVersion$, check = dGET
'   CASE: HEAD
'      ->vars:         "           check = dHEAD
'   CASE: POST
'      ->vars          "           check = dPOST
' - URI$:         here the adress (URI/URL) is saved
' - HTTPVersion$: This is the HTTP-Version
'----------------------- command line -------------------------------
' <command(GET,HEAD,POST)><Space><URI><Space><HTTP-Version><CrLf>
'********************************************************************************
sub request_line(var string header$; var byte check)
  long pos, ind, pos_next
  string commands$(16)
  statuscode$ = ""
  commands$ = " GET HEAD POST  "
  flag_tab$ = BFLAG_TAB$(KommandoTrenner, 0FFH, 0)
  result_check_keyword$ = check_keyword$(header$, flag_tab$, commands$, 0, 0)   ' search for command
  call getPosInd(result_check_keyword$, pos, ind)                               ' found? what position?
  if ind >= 0 then
    pos_next = scan_or_skip(header$, flag_tab$, pos + 1, 1)	' next Space
    URI$ = mid$(header$, pos + 1, pos_next - pos - 1)		' get URI(adress of document)
    call seperateVarListFromUri ( URI$ , VARLIST$ )             ' get parameters
    URI$ = trim$(URI$, "\/ ", 0)                                ' delete  /\ at the beginning and the end
    pos = pos_next                                              ' next pos						
    pos_next = scan_or_skip(header$, flag_tab$, pos + 1, 1)	' get next space
    HTTPVersion$ = mid$(header$, pos + 1, pos_next - pos - 3)	' get HTTP Version
    check = ind + 1                                             '
  else
    check = FALSE                                               ' no success
  endif
end

'***************************************************************************
' This sub seperates the variables from the uri
' the uri alone is saved in the old string URI$ and the variableList
' is saved in varList$
'***************************************************************************
sub seperateVarListFromUri(var string uri$,varList$)
  long pos
  pos = instr ( "?" , uri$ , 0 , 0 )                    ' searching for ?, because this seperates the parameters from the URI
  if pos > 0 then                                       ' found?
    varList$ = right$ ( uri$, len(uri$) - pos - 1 )     ' save variables
    uri$ = left$ ( uri$ , pos )                         ' save URI without variables
  endif
end

'*****************************************************************************+
' This is a not yet implemented function to get parameters from the URI
' They could be save in a second string to work with them. Variables are not
' that good, because we don't know haw much parameters will be send
'******************************************************************************
sub getURIParameters
end

'***********************************************************************
' This sub sends data in HTTP-protocoll. You only need the
' document to send, and the old data from request, but they are
' saved automaticly.
'-----------------------------------------------------------------------
' Parameters:
' document$: the answer document to send in entity
' content_type$: what kind of document
' typ:           what kind of request:
'                   1) dGET    --> complete answer
'                   2) dHEAD   --> only header will be send
' socket:        socket number, to which socket must this answer be send
' check:         was it successful?
'***********************************************************************
sub sendHTTP(var string document$, content_type$; byte typ; long socket; var byte check)
  long llTextToSendLen, llSentBytesNum

  if len(statuscode$) = 0 then              ' is there already a statuscode?
    if document$ = "" then                  ' is there a document to send or not?
      statuscode$ = HTTP_NOTFOUND           ' response an error code
    else
      statuscode$ = HTTP_OK                 ' all okay, there is a document
    endif
  endif

  answer$ = HTTP_version + statuscode$ + "<13><10>"		' status line

  if content_length = -1 then                                   ' not initialized for this document yet
    answer$ = answer$ + &
    HTTP_content_length + str$(len(document$)) + "<13><10>"	' length of document
  else
    answer$ = answer$ + &
    HTTP_content_length + str$(content_length)                  ' insert it to header
  endif

  if not document$ = "" then                                    ' is there a document
    if not content_type$ = "" then                              ' content-type saved?
      answer$ = answer$ + &
      HTTP_content_type + content_type$ + "<13><10>"		' insert type of document
    endif
  else
    if not content_type$ = "" then                              ' no content-type
      answer$ = answer$ + &
      HTTP_content_type + content_type$ + "<13><10>"		' type of document = empty
    endif
  endif
  answer$ = answer$ + HTTP_server			        ' add server to header
  ''answer$ = answer$ + "Connection: Keep-Alive<13><10>"
  answer$ = answer$ + "<13><10>"				' empty line

  llTextToSendLen = len(answer$)                                ' length for sending through ethernet
  #ifdef develop_debug_info
  if llTextToSendLen > maxHeaderSize then
    call textBox("maxHeaderSize zu klein")
  endif
  #endif
  call lSend( socket, answer$, llTextToSendLen, 0, llSentBytesNum )  ' send response
  if llSentBytesNum <> llTextToSendLen then                          ' how many bytes was send
    call textBox("Error: sending")                                   ' output to LCD
  endif

  if typ = dGET then                            ' really send entity when request = GET						
    if statuscode$ = HTTP_OK then               ' right statuscode or not?
      byte anz
      anz = 1
      'send document
      schleifenanfang:                          ' beginning of loop
        llTextToSendLen = len(document$)        ' length of document
        call lSend( socket, document$, llTextToSendLen, 0, llSentBytesNum ) ' send document
        if llSentBytesNum <> llTextToSendLen then ' how many bytes was send
          call textBox("Error: sending")          ' output to LCD
          check = FALSE                           ' NOT successful
          return
        endif
      if http_flags bitand 01H > 0 then           ' Flag was set?
        call getSendDocument(document$, URI$ , content_type$ , anz , check)   ' get document
        if check = FALSE then                                                 ' successful?
          call textBox("Error: get document")                                 ' output to LCD
        endif
        anz = anz + 1                                                         ' increase
        goto schleifenanfang                                                  ' end of loop
      endif
    endif
  endif

  statuscode$ = ""                          ' reset statuscode
end

'************************************************************************************
' this sub saves the requested document in answerDocument$. It's easy to get the
' the file from Flash, but the structure in flash have to be always the same.
' The startadresses of the files must be saved in the array filesAdr
'-------------------------------------------------------------------------------------------------
' Parameters:
'   s$:            here the document will be saved ( return value )
'   uri$:          URI ( URL ) of the document
'   content_type$: the content type of the document is saved in this variable
'   part:          var is needed for documents, which are bigger then the buffer
'   check = TRUE            ->  all done successfully
'   check = FALSE           ->  File not found
'   check = err_peek_flash  ->  error while reading flash memory(file found but not saved in string)
'************************************************************************************
sub getSendDocument(var string s$, uri$, content_type$; byte part ; var byte check)
  long ind, tmp
  flag_tab$ = BFLAG_TAB$(KommandoTrenner, 0FFH, 0)
  uri$ = trim$(uri$, "/", 0)                                              ' remove / at beginning and end
  result_check_keyword$ = check_keyword$(uri$, flag_tab$, files$, 0, 0)   ' search for files saved in flash memory
  call getPosInd(result_check_keyword$, tmp, ind)                         ' get index and position
  if ind >= 0 then                                                        ' file found?
    string fin$(10)                                                       ' end of name after "."
    ''tmp = filesAdr(ind + 1) - filesAdr(ind)
    check = TRUE
    if filesAdr(ind + 1) - filesAdr(ind) > max_len(s$) then               ' bigger than reserved memory in RAM?
      long adr
      long anz
      content_length = filesAdr(ind + 1) - filesAdr(ind)                  ' save real content length
      adr = filesAdr(ind) + ( part * max_len(s$) )                        ' save adress
      anz = limit( filesAdr(ind + 1) - filesAdr(ind) - ( part  * max_len(s$)) , 0 , max_len(s$) ) ' how much bytes to upload
      peek_flash adr, s$, anz, on_error_goto err                          ' read out from flash memory
      if adr + anz < filesAdr(ind + 1) then
        http_flags = http_flags bitor 01H		' not complete -> set bit
      else
        http_flags = http_flags bitand 0FEH             ' complete -> reset flag
      endif
    else
      peek_flash filesAdr(ind), s$, filesAdr(ind + 1) - filesAdr(ind), on_error_goto err   ' read out complete document
      http_flags = http_flags bitand 0FEH               ' reset flasg
    endif
    ' now we have to look up, which content Type the data is
    if URI$ = "" then
      fin$ = "html"
    else
      flag_tab$ = BFLAG_TAB$(".", 0FFH, 0)
      tmp = scan_or_skip(URI$, flag_tab$, 0, 1)		' search for point
      fin$ = right$(URI$, len(URI$) - tmp)              ' save name after the point
      fin$ = trim$ ( fin$ , "." , 1 )                   ' remove point(s)
    endif

    ' read out and set the current content type
    switch fin$
    case "html":
      content_type$ = html
    case "gif":
      content_type$ = gif
    case "jpg":
      content_type$ = jpg
    case "jpeg":
      content_type$ = jpeg
    case "png":
      content_type$ = png
    default:
      content_type$ = ""
    endswitch

  else
    content_type$ = ""
    check = FALSE                     ' not successful
  endif
  return                              ' exit function
  err:
  call err_peek_flash(check)          ' error while reading out of flash memory
end

